 -- ====================================================================
--
--
--
--
--
-- ====================================================================

IF EXISTS (SELECT
              *
           FROM
              SYSOBJECTS O, SYSUSERS U
           WHERE
              O.NAME = 'CUSTO_LANCAMENTO' AND
              TYPE = 'P' 		            AND
              U.UID = O.UID               AND
              U.NAME = USER)

DROP PROCEDURE CUSTO_LANCAMENTO
GO

CREATE PROCEDURE CUSTO_LANCAMENTO(
   @P_FILIMOVI        VARCHAR(04),
   @P_DATALANC        DATETIME,
   @P_CDALMOXARIFE    VARCHAR(06),
   @P_CDLOCALESTOQ    VARCHAR(06),
   @P_NRLOTEESTQ      VARCHAR(20),
   @P_NRSUBLOTE       VARCHAR(04),
   @P_CDPRODUTO       VARCHAR(10),
   @P_FILILANCTO      VARCHAR(04),
   @P_IDTIPOMOVI      VARCHAR(02),
   @P_VRCUSTOLANC     DOUBLE PRECISION OUTPUT,
   @P_VRCUSTLANCBR    DOUBLE PRECISION OUTPUT,
   @P_QTDELANCTO      DOUBLE PRECISION OUTPUT)
AS

   DECLARE  @V_QTDEESTO      DOUBLE PRECISION;
   DECLARE  @V_VALRESTO      DOUBLE PRECISION;
   DECLARE  @V_VALRBRUT      DOUBLE PRECISION;
   DECLARE  @V_QTDEANTE      DOUBLE PRECISION;
   DECLARE  @V_VRESTANT      DOUBLE PRECISION;
   DECLARE  @V_VRBRUANT      DOUBLE PRECISION;
   DECLARE  @V_QTDEESTOPOS   DOUBLE PRECISION;
   DECLARE  @V_VALRESTOPOS   DOUBLE PRECISION;
   DECLARE  @V_VALRBRUTPOS   DOUBLE PRECISION;
   DECLARE  @V_QTDEANTEPOS   DOUBLE PRECISION;
   DECLARE  @V_VRESTANTPOS   DOUBLE PRECISION;
   DECLARE  @V_VRBRUANTPOS   DOUBLE PRECISION;

   DECLARE  @V_CUSTPROD      DOUBLE PRECISION;

   DECLARE  @V_DTAUXILIAR    DATETIME;
   DECLARE  @V_IDTIPOMOVI    VARCHAR(02);
   DECLARE  @V_DTLANCMOVI    DATETIME;
   DECLARE  @V_VRLANCTOEST   DOUBLE PRECISION;
   DECLARE  @V_VRLANCTOBRUT  DOUBLE PRECISION;
   DECLARE  @V_QTLANCTOEST   DOUBLE PRECISION;
   DECLARE  @V_IDALTCMEDMV   VARCHAR(02);
   DECLARE  @V_CDFILIAL      VARCHAR(04);
   DECLARE  @V_NRLANCESTQ    VARCHAR(10);
   DECLARE  @V_NRSEQUITEM    VARCHAR(05);
   DECLARE  @V_COUNT         NUMERIC; 
   DECLARE  @V_IDORDEMCLC    VARCHAR(02);

BEGIN

      /* LER SALDO INICIAL DO PRODUTO */

   SELECT @V_QTDEESTOPOS = ROUND(ISNULL(MAX(QTESTOQDIA),   0), 5),
          @V_VALRESTOPOS = ROUND(ISNULL(MAX(VRESTOQDIA),   0), 5),
          @V_VALRBRUTPOS = ROUND(ISNULL(MAX(VRESTOQBRUT),  0), 5),
          @V_QTDEANTEPOS = ROUND(ISNULL(MAX(QTESTOQANT),   0), 5),
          @V_VRESTANTPOS = ROUND(ISNULL(MAX(VRESTOQANT),   0), 5),
          @V_VRBRUANTPOS = ROUND(ISNULL(MAX(VRBRUTOANT),   0), 5),
          @V_DTAUXILIAR  = MAX(DTPOSIESTQ),
          @V_COUNT       = COUNT(*)
   FROM POSESTDIA
   WHERE (CDFILIAL     = @P_FILIMOVI)          AND
         (CDALMOXARIFE = @P_CDALMOXARIFE)      AND
         (CDLOCALESTOQ = @P_CDLOCALESTOQ)      AND
         (NRLOTEESTQ   = @P_NRLOTEESTQ)        AND
         (NRSUBLOTE    = @P_NRSUBLOTE)         AND
         (CDPRODUTO    = @P_CDPRODUTO)         AND
         (DTPOSIESTQ   = @P_DATALANC+1);

   IF @V_COUNT = 0 BEGIN
      SELECT @V_QTDEESTOPOS = ROUND(ISNULL(MAX(QTESTOQDIA),   0), 5),
             @V_VALRESTOPOS = ROUND(ISNULL(MAX(VRESTOQDIA),   0), 5),
             @V_VALRBRUTPOS = ROUND(ISNULL(MAX(VRESTOQBRUT),  0), 5),
             @V_QTDEANTEPOS = ROUND(ISNULL(MAX(QTESTOQANT),   0), 5),
             @V_VRESTANTPOS = ROUND(ISNULL(MAX(VRESTOQANT),   0), 5),
             @V_VRBRUANTPOS = ROUND(ISNULL(MAX(VRBRUTOANT),   0), 5),
             @V_DTAUXILIAR  = MAX(DTPOSIESTQ),
             @V_COUNT       = COUNT(*)
      FROM POSESTDIA
      WHERE (CDFILIAL     = @P_FILIMOVI)          AND
            (CDALMOXARIFE = @P_CDALMOXARIFE)      AND
            (CDLOCALESTOQ = @P_CDLOCALESTOQ)      AND
            (NRLOTEESTQ   = @P_NRLOTEESTQ)        AND
            (NRSUBLOTE    = @P_NRSUBLOTE)         AND
            (CDPRODUTO    = @P_CDPRODUTO)         AND
            (DTPOSIESTQ   = (SELECT MAX(DTPOSIESTQ)
                             FROM POSESTDIA
                             WHERE (CDFILIAL     =  @P_FILIMOVI)     AND
                                   (CDALMOXARIFE =  @P_CDALMOXARIFE) AND
                                   (CDLOCALESTOQ =  @P_CDLOCALESTOQ) AND
                                   (NRLOTEESTQ   =  @P_NRLOTEESTQ)   AND
                                   (NRSUBLOTE    =  @P_NRSUBLOTE)    AND
                                   (CDPRODUTO    =  @P_CDPRODUTO)    AND
                                   (DTPOSIESTQ   < @P_DATALANC+1)));
   END;

   IF CONVERT(DATETIME,@P_DATALANC+1) <> @V_DTAUXILIAR BEGIN
      SET @V_QTDEESTO   = @V_QTDEESTOPOS;
      SET @V_VALRESTO   = @V_VALRESTOPOS;
      SET @V_VALRBRUT   = @V_VALRBRUTPOS;

      SET @V_QTDEANTE   = @V_QTDEANTEPOS;
      SET @V_VRESTANT   = @V_VRESTANTPOS;
      SET @V_VRBRUANT   = @V_VRBRUANTPOS;
   END
   ELSE BEGIN
      SET @V_QTDEESTO   = @V_QTDEANTEPOS;
      SET @V_VALRESTO   = @V_VRESTANTPOS;
      SET @V_VALRBRUT   = @V_VRBRUANTPOS;

      SET @V_QTDEANTE   = @V_QTDEANTEPOS;
      SET @V_VRESTANT   = @V_VRESTANTPOS;
      SET @V_VRBRUANT   = @V_VRBRUANTPOS;
   END;



   IF (@V_QTDEESTO <> 0) BEGIN
      SET @V_QTDEANTE   = @V_QTDEANTEPOS;
      SET @V_VRESTANT   = @V_VRESTANTPOS;
      SET @V_VRBRUANT   = @V_VRBRUANTPOS;
   END;

      SET @P_VRCUSTOLANC  = 0;
      SET @P_QTDELANCTO   = 0;
      SET @P_VRCUSTLANCBR = 0;

   DECLARE C_LANCTOEST CURSOR FOR
      SELECT IDTIPOMOVI, DTLANCMOVI, ISNULL(VRLANCTOEST,0) VRLANCTOEST,
             ISNULL(VRLANCTOBRUT, 0) VRLANCTOBRUT, QTLANCTOEST, IDALTCMEDMV,
             CDFILIAL, NRLANCESTQ, NRSEQUITEM, IDORDEMCLC
      FROM ITLANCTOEST
      WHERE (CDFILIMOVI   = @P_FILIMOVI)     AND
            (CDALMOXARIFE = @P_CDALMOXARIFE) AND
            (CDLOCALESTOQ = @P_CDLOCALESTOQ) AND
            (NRLOTEESTQ   = @P_NRLOTEESTQ)   AND
            (NRSUBLOTE    = @P_NRSUBLOTE)    AND
            (CDPRODMOVI   = @P_CDPRODUTO)    AND
            (DTLANCMOVI   = @P_DATALANC)     AND
            (ISNULL(IDEVOLESTOQ,'S') = 'S')
      ORDER BY DTLANCMOVI, ISNULL(IDORDEMCLC,IDTIPOMOVI), NRLANCESTQ, NRSEQUITEM;
   OPEN C_LANCTOEST;
   FETCH NEXT FROM C_LANCTOEST INTO @V_IDTIPOMOVI,   @V_DTLANCMOVI,  @V_VRLANCTOEST,
                                    @V_VRLANCTOBRUT, @V_QTLANCTOEST, @V_IDALTCMEDMV,
                                    @V_CDFILIAL,     @V_NRLANCESTQ,  @V_NRSEQUITEM, 
                                    @V_IDORDEMCLC;
   WHILE (@@FETCH_STATUS = 0) BEGIN

      IF (@V_QTDEESTO <> 0) BEGIN
         SET @V_QTDEANTE   = @V_QTDEESTO;
         SET @V_VRESTANT   = @V_VALRESTO;
         SET @V_VRBRUANT   = @V_VALRBRUT;
      END;

      /* TRATAMENTO PARA CADA REGISTRO DE LANAMENTO DE ESTOQUE */

      IF (@V_IDTIPOMOVI = '0') BEGIN    /* IMPLANTAO */

         SET @P_QTDELANCTO   = @V_QTLANCTOEST  - @V_QTDEESTO;
         SET @P_VRCUSTOLANC  = @V_VRLANCTOEST  - @V_VALRESTO;
         SET @P_VRCUSTLANCBR = @V_VRLANCTOBRUT - @V_VALRBRUT;

         SET @V_VALRBRUT    = @V_VRLANCTOBRUT;
         SET @V_VALRESTO    = @V_VRLANCTOEST;
         SET @V_QTDEESTO    = @V_QTLANCTOEST;
      END
      ELSE IF (@V_IDTIPOMOVI = '1') BEGIN    /* AJUSTES     */
         SELECT @V_CUSTPROD = TEKNISA.CUSTO_PRODUTO_LCEST(@P_FILIMOVI, @P_CDPRODUTO, '3', @P_DATALANC, @P_DATALANC,
                                                  @P_CDALMOXARIFE, @P_CDLOCALESTOQ, @P_NRLOTEESTQ, @P_NRSUBLOTE)

         IF @V_QTDEESTO <> 0 BEGIN
            SET @V_VALRBRUT = @V_VALRBRUT - ((@V_VALRBRUT/@V_QTDEESTO)* (@V_QTDEESTO - @V_QTLANCTOEST));
            SET @V_VALRESTO = @V_VALRESTO - ((@V_VALRESTO/@V_QTDEESTO)* (@V_QTDEESTO - @V_QTLANCTOEST));
         END
         ELSE BEGIN
            IF (@V_QTDEESTOPOS <> 0) BEGIN
               SET @V_VALRBRUT = ((@V_VALRBRUT/@V_QTDEESTOPOS)* @V_QTLANCTOEST);
               SET @V_VALRESTO = ((@V_VALRESTOPOS/@V_QTDEESTOPOS)* @V_QTLANCTOEST);
            END  
            ELSE BEGIN
               SET @V_VALRBRUT = 0;
               SET @V_VALRESTO = 0;
            END;
         END;
         IF  ISNULL(@V_IDORDEMCLC, @V_IDTIPOMOVI) = '0' BEGIN
            SET @P_QTDELANCTO   = @V_QTLANCTOEST  -  @V_QTDEESTO;
            SET @P_VRCUSTOLANC  = (@V_QTLANCTOEST  - @V_QTDEESTO) * @V_CUSTPROD;
            SET @P_VRCUSTLANCBR = (@V_QTLANCTOEST  - @V_QTDEESTO) * @V_CUSTPROD;
         END
         ELSE BEGIN
            SET @P_QTDELANCTO   = @V_QTLANCTOEST;
            SET @P_VRCUSTOLANC  = @V_VALRESTO;
            SET @P_VRCUSTLANCBR = @V_VALRBRUT;
         END;
      END
      ELSE IF (@V_IDTIPOMOVI = '2') BEGIN    /* ENTRADA */

         SET @V_QTDEESTO     = @V_QTDEESTO   +  @V_QTLANCTOEST;

         IF @V_QTLANCTOEST = 0 BEGIN
            SET @V_VALRBRUT = @V_VALRBRUT + @V_VRLANCTOBRUT;
            SET @V_VALRESTO = @V_VALRESTO + @V_VRLANCTOEST;
         END
         ELSE BEGIN
            SET @V_VALRBRUT = @V_VALRBRUT + (@V_QTLANCTOEST*@V_VRLANCTOBRUT);
            SET @V_VALRESTO = @V_VALRESTO + (@V_QTLANCTOEST*@V_VRLANCTOEST);
         END;

         IF @V_QTDEESTO  = 0 BEGIN
            SET @V_VALRBRUT   = 0;
            SET @V_VALRESTO   = 0;
         END;
         SET @P_VRCUSTOLANC  = @V_VRLANCTOEST;
         SET @P_QTDELANCTO   = @V_QTLANCTOEST;
         SET @P_VRCUSTLANCBR = @V_VRLANCTOBRUT;

      END
      ELSE IF (@V_IDTIPOMOVI = '3') BEGIN    /* RETIRADA    */
         IF @V_QTDEESTO > 0 BEGIN
            SET @V_VALRBRUT = @V_VALRBRUT - ((@V_VALRBRUT/@V_QTDEESTO)*@V_QTLANCTOEST);
            SET @V_VALRESTO = @V_VALRESTO - ((@V_VALRESTO/@V_QTDEESTO)*@V_QTLANCTOEST);
         END;
         SET @V_QTDEESTO = @V_QTDEESTO - @V_QTLANCTOEST;

         IF (@V_QTDEESTO < 0) BEGIN
            IF (@V_QTDEANTE <> 0) BEGIN
               SET @V_VALRESTO = ((@V_VRESTANT/@V_QTDEANTE)*@V_QTDEESTO);
               SET @V_VALRBRUT = ((@V_VRBRUANT/@V_QTDEANTE)*@V_QTDEESTO);
            END
            ELSE BEGIN
               SET @V_VALRESTO = 0;
               SET @V_VALRBRUT = 0;
            END;
         END;

         IF ROUND(@V_QTDEESTO,3) = 0 BEGIN
            SET @V_QTDEESTO = 0;
         END;

         IF @V_QTDEESTO  = 0 BEGIN
            SET @V_VALRBRUT   = 0;
            SET @V_VALRESTO   = 0;
         END;
      END
      ELSE IF (@V_IDTIPOMOVI = '4') BEGIN    /* SADA       */

         IF @V_QTDEESTO > 0 BEGIN
            SET @V_VALRBRUT   = @V_VALRBRUT - ((@V_VALRBRUT/@V_QTDEESTO)*@V_QTLANCTOEST);
            SET @V_VALRESTO   = @V_VALRESTO - ((@V_VALRESTO/@V_QTDEESTO)*@V_QTLANCTOEST);
         END;

         SET @V_QTDEESTO = @V_QTDEESTO - @V_QTLANCTOEST;

         IF (@V_QTDEESTO < 0) BEGIN
            IF (@V_QTDEANTE <> 0) BEGIN
               SET @V_VALRESTO = ((@V_VRESTANT/@V_QTDEANTE)*@V_QTDEESTO);
               SET @V_VALRBRUT = ((@V_VRBRUANT/@V_QTDEANTE)*@V_QTDEESTO);
            END
            ELSE BEGIN
               SET @V_VALRESTO = 0;
               SET @V_VALRBRUT = 0;
            END;
         END;

         IF ROUND(@V_QTDEESTO,3) = 0 BEGIN
            SET @V_QTDEESTO = 0;
         END;

         IF @V_QTDEESTO  = 0 BEGIN
            SET @V_VALRBRUT  = 0;
            SET @V_VALRESTO  = 0;
         END;
      END
      ELSE IF (@V_IDTIPOMOVI = '5') BEGIN    /* DEV. FORN.  */

         IF @V_QTLANCTOEST = 0 BEGIN
            SET @V_VALRBRUT = @V_VALRBRUT - @V_VRLANCTOBRUT;
            SET @V_VALRESTO = @V_VALRESTO - @V_VRLANCTOEST;
         END
         ELSE BEGIN
            SET @V_CUSTPROD = 0;
            IF @V_QTDEESTO <> 0 BEGIN
               SET @V_CUSTPROD = @V_VALRBRUT / @V_QTDEESTO;
            END
            ELSE BEGIN
               IF @V_QTDEANTE <> 0 BEGIN
                  SET @V_CUSTPROD = @V_VRBRUANT / @V_QTDEANTE;
               END;
            END;

            IF (@V_VALRBRUT - (@V_QTLANCTOEST*@V_VRLANCTOBRUT)) >= 0 BEGIN
               SET @V_VALRBRUT     = @V_VALRBRUT - (@V_QTLANCTOEST*@V_VRLANCTOBRUT);
               SET @P_VRCUSTLANCBR = @V_QTLANCTOEST * @V_VRLANCTOBRUT;
            END
            ELSE BEGIN
               SET @V_VALRBRUT     = @V_VALRBRUT - (@V_QTLANCTOEST*@V_CUSTPROD);
               SET @P_VRCUSTLANCBR = @V_QTLANCTOEST * @V_CUSTPROD;
            END;

            SET @V_CUSTPROD = 0;
            IF @V_QTDEESTO <> 0 BEGIN
               SET @V_CUSTPROD = @V_VALRESTO / @V_QTDEESTO;
            END
            ELSE BEGIN
               IF @V_QTDEANTE <> 0 BEGIN
                  SET @V_CUSTPROD = @V_VRESTANT / @V_QTDEANTE;
               END;
            END;

            IF (@V_VALRESTO - (@V_QTLANCTOEST*@V_VRLANCTOEST)) >= 0 BEGIN
               SET @V_VALRESTO     = @V_VALRESTO - (@V_QTLANCTOEST*@V_VRLANCTOEST);
               SET @P_VRCUSTOLANC  = @V_QTLANCTOEST * @V_VRLANCTOEST;
            END
            ELSE BEGIN
               SET @V_VALRESTO     = @V_VALRESTO - (@V_QTLANCTOEST*@V_CUSTPROD);
               SET @P_VRCUSTOLANC  = @V_QTLANCTOEST * @V_CUSTPROD;
            END;
         END;

         SET @V_QTDEESTO = @V_QTDEESTO - @V_QTLANCTOEST;

         IF ROUND(@V_QTDEESTO,3) = 0 BEGIN
            SET @V_QTDEESTO = 0;
         END;

         IF @V_QTDEESTO  = 0 BEGIN
            SET @V_VALRBRUT   = 0;
            SET @V_VALRESTO   = 0;
         END;

         SET @P_QTDELANCTO   = @V_QTLANCTOEST  - @V_QTDEESTO;
      END;

      /* GUARDA OS VALORES DE ESTOQUE ANTERIORES  PRXIMA MOVIMENTAO */
      SET @V_QTDEESTOPOS   = @V_QTDEESTO;
      SET @V_VALRESTOPOS   = @V_VALRESTO;
      SET @V_VALRBRUTPOS   = @V_VALRBRUT;
      SET @V_QTDEANTEPOS   = @V_QTDEANTE;
      SET @V_VRESTANTPOS   = @V_VRESTANT;
      SET @V_VRBRUANTPOS   = @V_VRBRUANT;

      IF @V_IDTIPOMOVI  = @P_IDTIPOMOVI  BEGIN
         BREAK;
      END;
      FETCH NEXT FROM C_LANCTOEST INTO @V_IDTIPOMOVI,   @V_DTLANCMOVI,  @V_VRLANCTOEST,
                                       @V_VRLANCTOBRUT, @V_QTLANCTOEST, @V_IDALTCMEDMV,
                                       @V_CDFILIAL,     @V_NRLANCESTQ,  @V_NRSEQUITEM,
                                       @V_IDORDEMCLC;
   END;
   CLOSE C_LANCTOEST;
   DEALLOCATE C_LANCTOEST;
END;
GO



-- VERSAO --
EXEC ATUVEROBJ 'CUSTO_LANCAMENTO','5.03.009';
GO

